#ifndef f_VECTOR_H
#define f_VECTOR_H

#include <math.h>

class Vector3 {
public:
	double x, y, z;

	Vector3() {}
	Vector3(double _x, double _y, double _z) : x(_x), y(_y), z(_z) {}

	const Vector3& operator+() const { return *this; }
	Vector3 operator-() const { return Vector3(-x,-y,-z); }

	Vector3 operator+(const Vector3& v) const { return Vector3(x+v.x, y+v.y, z+v.z); }
	Vector3 operator-(const Vector3& v) const { return Vector3(x-v.x, y-v.y, z-v.z); }
	const Vector3& operator+=(const Vector3& v) { x+=v.x; y+=v.y; z+=v.z; return *this; }
	const Vector3& operator-=(const Vector3& v) { x-=v.x; y-=v.y; z-=v.z; return *this; }
	const Vector3& operator*=(double r) { x*=r; y*=r; z*=r; return *this; }
	double operator*(const Vector3& v) const { return x*v.x + y*v.y + z*v.z; }
	Vector3 operator*(double r) const { return Vector3(x*r, y*r, z*r); }
	Vector3 operator%(const Vector3& v) const;

	Vector3 norm() const;
	double len() const;
	double len2() const { return x*x + y*y + z*z; }

	double dist(Vector3& v) { return operator-(v).len(); }

	Vector3 xy() const { return Vector3(x,y,0); }

	Vector3 rotate2dX(double cs, double sn) const;
	Vector3 rotate2dY(double cs, double sn) const;
	Vector3 rotate2dZ(double cs, double sn) const;

	Vector3	rotate2dX(double t) { return rotate2dX(cos(t), sin(t)); }
	Vector3	rotate2dY(double t) { return rotate2dY(cos(t), sin(t)); }
	Vector3	rotate2dZ(double t) { return rotate2dZ(cos(t), sin(t)); }
};

inline Vector3 operator*(double r, const Vector3& v) { return v*r; }

///////////////////////////////////////////////////////////////////////////

class Quaternion {
public:
	Vector3 xyz;
	double w;

	Quaternion() {}
	Quaternion(double angle, const Vector3& axis) throw() : xyz(axis * sin(angle*0.5)), w(cos(angle*0.5)) {}
	Quaternion(const Vector3& _xyz, double _w) throw() : xyz(_xyz), w(_w) {}
	Quaternion(const Vector3& v1, const Vector3& v2) throw();

	Quaternion operator-() const throw() { return Quaternion(-xyz, -w); }
	Quaternion operator~() const throw() { return Quaternion(-xyz, w); }
	const Quaternion& operator*=(const Quaternion& r) throw() {
		Quaternion t(xyz % r.xyz + r.w*xyz + w*r.xyz, w*r.w - xyz*r.xyz);

		return *this = t;
	}
	Quaternion operator*(const Quaternion& q) const throw() { return Quaternion(Quaternion(*this)*=q); }

	const Quaternion& RotateX(double angle) throw() { return operator*=(Quaternion(Vector3(sin(angle*0.5), 0.0, 0.0), cos(angle*0.5))); }
	const Quaternion& RotateY(double angle) throw() { return operator*=(Quaternion(Vector3(0.0, sin(angle*0.5), 0.0), cos(angle*0.5))); }
	const Quaternion& RotateZ(double angle) throw() { return operator*=(Quaternion(Vector3(0.0, 0.0, sin(angle*0.5)), cos(angle*0.5))); }
	Vector3 RotatePoint(const Vector3& pt) const throw();

	Quaternion slerp(const Quaternion& dst, double t) const throw();
	Quaternion slerp(double t) const throw();

	void ToMatrix(double *dst) const throw();

	void Normalize() throw();
};


#endif
